#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const minimist = require("minimist");
const api_1 = require("./api");
const util_1 = require("util");
const packageJson = require("../package.json");
//#endregion
const deprecated = (what, now) => util_1.deprecate(() => void 0, `--${what} is deprecated. Use --${now} instead.`)();
exports.getFileNameInTS = (base, name, dec, tsx) => {
    return path_1.join(base, name + (dec ? '.d' : '') + '.ts' + (tsx ? 'x' : ''));
};
function CLIMain(_config) {
    if (_config.in) {
        deprecated('in', 'schema');
        _config.schemas = [_config.in];
    }
    if (_config.dryrun) {
        deprecated('dryrun', 'noEmit');
        _config.noEmit = true;
    }
    if (_config.templateUrl) {
        deprecated('templateUrl', 'template');
    }
    _config.schemas.forEach(async (file) => {
        const config = {
            schema: file,
            isSchemaUrl: true,
            template: _config.template,
            isTemplateUrl: true,
            declaration: _config.declaration,
            customFileComment: _config.customFileComment,
        };
        const result = await api_1.default(config);
        if (_config.noEmit) {
            return;
        }
        const inputName = path_1.parse(file).name;
        const outputPath = _config.outPath
            ? exports.getFileNameInTS(_config.outPath, inputName, _config.declaration, _config.tsx)
            : undefined;
        await util_1.promisify(fs_1.writeFile)(outputPath || _config.out || exports.getFileNameInTS('./', inputName, _config.declaration, _config.tsx), result);
    });
}
exports.default = CLIMain;
if (require.main === module) {
    const c = Object.assign({}, minimist(process.argv.slice(2)));
    if (c.h)
        c.help = c.h;
    if (c.d)
        c.declaration = c.d;
    if (c.o)
        c.out = c.o;
    if (c._)
        c.schemas = c._;
    //#region Help
    if (c.help) {
        console.log(`Schema2ts ${packageJson.version} (${packageJson.repository})
${packageJson.description}

Syntax: schema2ts [options] [file ...]

Options:
    -h, --help                   Print this message.
    --template FILE/URL          Template of the generated files.
    --tsx                        Generates '.tsx' file (or .d.tsx if -d is on).
    -d, --declaration            Generates corresponding '.d.ts' file.
    -o, --out FILE               Generates file to someplace.
    --outPath DIRECTORY          Generates file to the directory.
    --noEmit                     Do not emit outputs.
`);
    }
    else {
        //#endregion
        CLIMain(c);
    }
    process.on('unhandledRejection', (reason, p) => {
        throw reason;
    });
}
//# sourceMappingURL=cli.js.map