"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const utils_1 = require("../utils");
const packageJson = require("../../package.json");
const ast = require("ts-simple-ast");
class Render extends ast.default {
    constructor(config = {}) {
        super({
            compilerOptions: {
                declaration: config.declarationOnly,
                target: ts.ScriptTarget.Latest,
                module: ts.ModuleKind.ESNext,
                moduleResolution: ts.ModuleResolutionKind.NodeJs,
            },
            manipulationSettings: {
                quoteType: ast.QuoteType.Single,
            },
        }, new utils_1.ReadonlyFileSystemHost());
        this.config = config;
    }
    schema2tsEmit(which) {
        const sourcefile = this.getSourceFile(which);
        if (!sourcefile) {
            throw new Error('File not found when emitting.');
        }
        const dts = sourcefile.emit({ emitOnlyDtsFiles: this.config.declarationOnly });
        dts.getDiagnostics().forEach(diag => console.warn(`TS${diag.getCode()}: ${diag.getMessageText()}`));
        if (dts.getEmitSkipped()) {
            throw new Error('Typescript skipped emit file.');
        }
        let path = sourcefile.getFilePath();
        if (this.config.declarationOnly) {
            path = path.replace(/\.ts$/, '.d.ts');
        }
        return this.getFileSystem().readFileSync(path, 'utf-8');
    }
    static injectTemplateVariables(str) {
        const now = new Date();
        return str
            .replace(/%version%/g, packageJson.version)
            .replace(/%when%/g, now.toLocaleDateString() + ' ' + now.toLocaleTimeString())
            .replace(/%typescript-version%/g, packageJson.dependencies ? packageJson.dependencies.typescript : 'unknown');
    }
}
Render.nodeToString = (() => {
    const printer = ts.createPrinter({ newLine: ts.NewLineKind.LineFeed, removeComments: false });
    const doc = ts.createSourceFile('', '', ts.ScriptTarget.Latest);
    return (node) => printer.printNode(ts.EmitHint.Unspecified, node, doc);
})();
exports.Render = Render;
//# sourceMappingURL=render.js.map